@@ -5,12 +5,13 @@
  *
  * Updated with the help of Dick Streefland, Mark Lord, Doug
  * L. Hoffman (hoffman@cs.unc.edu), Ben Galliart (bgallia@luc.edu),
- * Markus Pilzecker (markus.pilzecker@rhein-neckar.netsurf.de), and
- * Donnie Barnes (djb@redhat.com)
+ * Markus Pilzecker (markus.pilzecker@rhein-neckar.netsurf.de), 
+ * Donnie Barnes (djb@redhat.com), Grant Guenther (grant@torque.net),
+ * and Erik Troan (ewt@redhat.com).
  *
  ********************************************************************
  *
- * Copyright (C) 1994-96 Jeff Tranter (jeff_tranter@pobox.com)
+ * Copyright (C) 1994-97 Jeff Tranter (jeff_tranter@pobox.com)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,10 +44,12 @@
 #include <string.h>
 #include <sys/mount.h>
 #include <linux/cdrom.h>
+#include <linux/ucdrom.h>
 #include <sys/stat.h>
 #include <linux/fd.h>
 #include <sys/sysmacros.h>
 #include <linux/major.h>
+#include <sys/wait.h>
 
 /* set the default device here */
 char default_device[1024];
@@ -73,38 +76,48 @@ typedef struct {
 
 /* table of device nicknames */
 nickname_t nickname[] = { 
+  { "cd0",    "/dev/cd0" },
+  { "cd1",    "/dev/cd1" },
   { "cdrom",  "/dev/cdrom" },
+  { "cdu31a", "/dev/cdu31a" },
+  { "cdu535", "/dev/cdu535" },
+  { "fd0",    "/dev/fd0" },
+  { "floppy", "/dev/fd0" },
+  { "hdb",    "/dev/hdb" },
+  { "hdc",    "/dev/hdc" },
+  { "hdd",    "/dev/hdd" },
+  { "jaz",    "/dev/jaz" },
+  { "lmscd",  "/dev/lmscd" },
+  { "mcd",    "/dev/mcd" },
   { "sbpcd" , "/dev/sbpcd" },
   { "sbpcd0", "/dev/sbpcd0" },
   { "sbpcd1", "/dev/sbpcd1" },
   { "sbpcd2", "/dev/sbpcd2" },
   { "sbpcd3", "/dev/sbpcd3" },
-  { "cdu31a", "/dev/cdu31a" },
-  { "cdu535", "/dev/cdu535" },
-  { "mcd",    "/dev/mcd" },
-  { "lmscd",  "/dev/lmscd" },
   { "scd0",   "/dev/scd0" },
   { "scd1",   "/dev/scd1" },
-  { "cd0",    "/dev/cd0" },
-  { "cd1",    "/dev/cd1" },
   { "sr0",    "/dev/sr0" },
   { "sr1",    "/dev/sr1" },
   { "sra",    "/dev/sra" },
   { "srb",    "/dev/srb" },
-  { "hdb",    "/dev/hdb" },
-  { "hdc",    "/dev/hdc" },
-  { "hdd",    "/dev/hdd" },
-  { "fd0",    "/dev/fd0" },
-  { "floppy", "/dev/fd0" },
+  { "zip",    "/dev/zip" },
   /* next entry indicates end of table */
   { 0, 0 }
 };
 
+#define SCSI_IOCTL_SEND_COMMAND 1
+
+struct sdata {
+  int  inlen;
+  int  outlen;
+  char cmd[256];
+} scsi_cmd;
+
 /* display command usage on standard error and exit */
 void usage()
 {
   fprintf(stderr,
-"Eject version 1.4 by Jeff Tranter (jeff_tranter@pobox.com)\n"
+"Eject version 1.5 by Jeff Tranter (jeff_tranter@pobox.com)\n"
 "\n"
 "Usage:\n"
 "  eject -h                           -- display this screen and exit\n"
@@ -151,9 +164,11 @@ void parse_args(int argc, char **argv)
 	a_arg = 0;
       else if (!strcmp(optarg, "off"))
 	a_arg = 0;
-      else if (!strcmp(optarg, "ferme")) /* pardon my French */
+      else if (!strcmp(optarg, "ferme"))     /* pardon my French */
 	a_arg = 0;
-      else if (!strcmp(optarg, "aus"))
+      else if (!strcmp(optarg, "aus"))       /* German */
+	a_arg = 0;
+      else if (!strcmp(optarg, "disattiva")) /* Italian */
 	a_arg = 0;
       else if (!strcmp(optarg, "1"))
 	a_arg = 1;
@@ -163,6 +178,8 @@ void parse_args(int argc, char **argv)
 	a_arg = 1;
       else if (!strcmp(optarg, "auf"))
 	a_arg = 1;
+      else if (!strcmp(optarg, "attiva"))
+	a_arg = 1;
       else
 	usage();
       break;
@@ -190,6 +207,69 @@ void parse_args(int argc, char **argv)
     strncpy(device, argv[optind], sizeof(device)-1); /* one argument */
 }
 
+/* determine if device is a jaz...returns 1 if true */
+int
+is_jaz( int fd )
+{
+  char  id[25];
+  int   i;
+
+  scsi_cmd.inlen = 0;
+  scsi_cmd.outlen = 40;
+  scsi_cmd.cmd[0] = 0x12;               /* inquiry */
+  scsi_cmd.cmd[1] = 0;
+  scsi_cmd.cmd[2] = 0;
+  scsi_cmd.cmd[3] = 0;
+  scsi_cmd.cmd[4] = 40;
+  scsi_cmd.cmd[5] = 0;
+
+  if (ioctl(fd,SCSI_IOCTL_SEND_COMMAND,(void *)&scsi_cmd))
+    return 0;  /* if the SCSI test fails, it is certainly not a JAZ or ZIP */
+
+  for(i=0;i<24;i++) {
+    id[i] = scsi_cmd.cmd[i+8];
+  }
+  id[24] = 0;
+
+  if (!strncasecmp(id,"IOMEGA  JAZ 1GB",15) || !strncasecmp(id,"IOMEGA  ZIP",11)) 
+  return(1);
+  else return(0);
+}
+
+/* send command to SCSI device */
+void
+motor( int fd, int mode )
+{
+  scsi_cmd.inlen = 0;
+  scsi_cmd.outlen = 0;
+  scsi_cmd.cmd[0] = 0x1b;               /* start/stop */
+  scsi_cmd.cmd[1] = 0;
+  scsi_cmd.cmd[2] = 0;
+  scsi_cmd.cmd[3] = 0;
+  scsi_cmd.cmd[4] = mode;
+  scsi_cmd.cmd[5] = 0;
+
+  if (ioctl(fd,SCSI_IOCTL_SEND_COMMAND,(void *)&scsi_cmd))
+    perror("eject: motor control ioctl error");
+}
+
+/* unlock the door before ejecting */
+void
+unlockdoor( int fd )
+{
+  scsi_cmd.inlen = 0;
+  scsi_cmd.outlen = 0;
+  scsi_cmd.cmd[0] = 0x1e;               /* prevent/allow media removal */
+  scsi_cmd.cmd[1] = 0;
+  scsi_cmd.cmd[2] = 0;
+  scsi_cmd.cmd[3] = 0;
+  scsi_cmd.cmd[4] = 0;
+  scsi_cmd.cmd[5] = 0;
+
+  if (ioctl(fd,SCSI_IOCTL_SEND_COMMAND,(void *)&scsi_cmd))
+    perror("eject: door unlock ioctl error");
+}
+
 int
 main(int argc, char **argv)
 {
@@ -200,9 +280,10 @@ main(int argc, char **argv)
   char s1[1024] = "";            /* for reading /etc/mtab */
   char s2[1024] = "";            /* "   "       "         */
   char dummy[256];               /* "   "       "         */
-  char umount_cmd[1024] = "/bin/umount "; /* umount command string */
   char *s3;                      /* general purpose */
   FILE *fp;                      /* for reading /etc/mtab */
+  char * umountDevStack[20];	 /* eww -- a hard limit */
+  int umountDevPtr = 0;
 
   /* get default device from $CDROM environment variable; if not set
      then use compiled in default */
@@ -228,7 +309,7 @@ main(int argc, char **argv)
   if (n_option) {
     printf("eject: device nicknames:\n");
     printf("Nickname  Device\n");
-    printf("--------  ----------\n");
+    printf("--------  ------\n");
     for (i = 0 ; nickname[i].name != 0 ; i++)
       printf("%-8s  %-8s\n", nickname[i].name, nickname[i].device);
     exit(0);
@@ -296,9 +377,21 @@ main(int argc, char **argv)
     exit(0);
   }
 
-  /* handle IDE/ATAPI CDROM changer option */
+  /* Handle IDE/ATAPI CDROM changer option. CDROM_SELECT_DISC is
+   * preferred, older kernels used CDROMLOADFROMSLOT */
+  
   if (c_option) {
-#ifdef CDROMLOADFROMSLOT
+#ifdef CDROM_SELECT_DISC
+    if (v_option)
+      printf("eject: switch to CDROM slot #%i\n", c_arg);
+    status = ioctl(fd, CDROM_SELECT_DISC, c_arg);
+    if (status != 0) {
+      sprintf(s, "eject: CDROM_SELECT_DISC ioctl failed for `%s' (slot %i)", 
+        device, c_arg);
+      perror(s);
+      exit(1);
+    }
+#elif defined CDROMLOADFROMSLOT
     if (v_option)
       printf("eject: switch to CDROM slot #%i\n", c_arg);
     status = ioctl(fd, CDROMLOADFROMSLOT, c_arg);
@@ -315,11 +408,14 @@ main(int argc, char **argv)
   }
   
   /* see if device has been mounted by looking in /etc/mtab */
-  fp = fopen("/etc/mtab", "r");
-  while (!feof(fp)) {
-    fscanf(fp, "%s %s %s %s %s %s", s1, s2, dummy, dummy, dummy, dummy);
-    if (!strcmp(s1, device)) {
-      if (!f_option) {
+  #ifdef __linux__
+    fp = fopen("/proc/mounts", "r");
+  #else
+    fp = fopen("/etc/mtab", "r");
+  #endif
+  while (fscanf(fp, "%s %s %s %s %s %s", s1, s2, dummy, dummy, dummy, dummy) >= 4) {
+    if (!strncmp(s1, device,8)) {
+      if (!f_option && !u_option) {
 	printf("eject: `%s' is mounted at `%s', not ejected\n", s1, s2);
 	exit(1);
       } else {
@@ -327,19 +423,32 @@ main(int argc, char **argv)
 	  printf("eject: `%s' is mounted at `%s'\n", s1, s2);
 	if (u_option) {
 	  if (v_option)
-	    printf("eject: trying to unmount `%s'\n", s1);
-	  strcat(umount_cmd, s1);      /* setup umount command */
-	  status = system(umount_cmd); /* do it */
-	  if ((status == -1) || (status == 127)) {
-	    sprintf(s, "eject: unmount of `%s' failed", s1);
-	    perror(s);
-	  }
+	    printf("eject: will unmount `%s' a bit later\n", s1);
+	  umountDevStack[umountDevPtr++] = strdup(s1);
 	}
-      }
-      break;
+     }
+   }
+  }
+
+  fclose(fp);
+
+  while (umountDevPtr--) {
+    if (v_option)
+      printf("eject: trying to unmount `%s'\n", umountDevStack[umountDevPtr]);
+    if (!fork()) {
+	  execl("/bin/umount", "/bin/umount", umountDevStack[umountDevPtr], 
+			NULL);
+	  fprintf(stderr, "unable to exec /bin/umount of %s: %s",
+		  umountDevStack[umountDevPtr], strerror(errno));
+	  exit(1);
+    }
+    wait(&status);
+
+    if (!WIFEXITED(status)) {
+      sprintf(s, "eject: unmount of `%s' failed", umountDevStack[umountDevPtr]);
+      perror(s);
     }
   }
-  fclose(fp);
   
   fstat(fd, &mystat);
   if (major(mystat.st_rdev) == FLOPPY_MAJOR) {
@@ -350,8 +459,12 @@ main(int argc, char **argv)
       perror(s);
       exit(1);
     }
+  }  else if (is_jaz(fd))  {
+        unlockdoor(fd);
+        motor(fd,1);
+        motor(fd,2);
   }  else {
-    /* if we're not a floppy, we must be a CD */
+    /* if we're not a floppy or a jaz, we must be a CD */
     status = ioctl(fd, CDROMEJECT);
     if (status != 0) {
       sprintf(s, "eject: CDROMEJECT ioctl failed for `%s'", device);
